@@ -7,6 +7,7 @@ Created on Fri Nov 13 11:30:24 2020
 """
 
 import numpy as np
+import math
 import tensorflow.keras as keras
 
 
@@ -267,6 +268,8 @@ def export_to_meme(logos, name, alphabet):
                 for num in line:
                     if num == 0:
                         f.write('0.0\t')
+                    elif math.isnan(num):
+                        f.write('0.25\t')
                     else:
                         f.write(str(num)+'\t')
 
